package ru.myitschool.work.ui.main

import android.graphics.BitmapFactory
import android.os.Bundle
import android.util.Log
import android.view.View
import androidx.fragment.app.Fragment
import androidx.fragment.app.viewModels
import androidx.navigation.fragment.findNavController
import dagger.hilt.android.AndroidEntryPoint
import kotlinx.coroutines.CoroutineScope
import kotlinx.coroutines.Dispatchers
import kotlinx.coroutines.launch
import kotlinx.coroutines.withContext
import ru.myitschool.work.R
import ru.myitschool.work.databinding.MainFragmentBinding
import ru.myitschool.work.ui.login.LoginDestination
import ru.myitschool.work.ui.qr.scan.QrScanDestination
import ru.myitschool.work.utils.AuthPreferences
import ru.myitschool.work.utils.collectWhenStarted
import java.net.URL
import java.util.Locale

@AndroidEntryPoint
class MainFragment : Fragment(R.layout.main_fragment) {
    private var _binding: MainFragmentBinding? = null
    private val binding: MainFragmentBinding get() = _binding!!
    private lateinit var authPreferences: AuthPreferences
    private val viewModel: MainViewModel by viewModels()

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        authPreferences = AuthPreferences(requireContext())

        if (!authPreferences.isLoggedIn()) {
            navigateToLogin()
        }
    }

    override fun onViewCreated(view: View, savedInstanceState: Bundle?) {
        super.onViewCreated(view, savedInstanceState)
        _binding = MainFragmentBinding.bind(view)

        if (authPreferences.isLoggedIn()) {
            setupMainComponents()
            observeUserState()
            viewModel.getUserData(authPreferences.getLogin() ?: "")
        }
    }

    private fun hideAll() {
        binding.apply {
            username.visibility = View.GONE
            position.visibility = View.GONE
            lastEntry.visibility = View.GONE
            scan.visibility = View.GONE
            logout.visibility = View.GONE

            error.visibility = View.VISIBLE
            refresh.visibility = View.VISIBLE
        }
    }

    private fun observeUserState() {
        viewModel.state.collectWhenStarted(this) { state ->
            when (state) {
                is MainViewModel.MainState.Initial -> {
                    binding.error.visibility = View.GONE
                }
                is MainViewModel.MainState.Loading -> {
                    binding.error.visibility = View.GONE
                }
                is MainViewModel.MainState.Success -> {
                    binding.apply {
                        username.visibility = View.VISIBLE
                        position.visibility = View.VISIBLE
                        lastEntry.visibility = View.VISIBLE
                        scan.visibility = View.VISIBLE
                        logout.visibility = View.VISIBLE
                        error.visibility = View.GONE

                        username.text = state.userInfo.name
                        position.text = state.userInfo.position

                        loadImageFromUrl(state.userInfo.photoUrl)

                        val inputFormat = java.text.SimpleDateFormat("yyyy-MM-dd'T'HH:mm:ss", Locale.getDefault())
                        val outputFormat = java.text.SimpleDateFormat("yyyy-MM-dd HH:mm", Locale.getDefault())

                        try {
                            val parsedDate = inputFormat.parse(state.userInfo.lastVisit)
                            lastEntry.text = parsedDate?.let { outputFormat.format(it) } ?: state.userInfo.lastVisit
                        } catch (e: Exception) {
                            lastEntry.text = state.userInfo.lastVisit
                        }
                    }
                }
                is MainViewModel.MainState.Error -> {
                    hideAll()
                    binding.error.text = state.message
                }
            }
        }
    }

    private fun loadImageFromUrl(url: String) {
        CoroutineScope(Dispatchers.IO).launch {
            val bitmap = try {
                val input = URL(url).openStream()
                BitmapFactory.decodeStream(input)
            } catch (e: Exception) {
                Log.e("MainFragment", "Error loading image", e)
                null
            }

            withContext(Dispatchers.Main) {
                if (bitmap != null) {
                    binding.photo.setImageBitmap(bitmap)
                } else {
                    binding.photo.setImageResource(R.drawable.ic_no_img)
                }
            }
        }
    }

    private fun setupMainComponents() {
        binding.logout.setOnClickListener {
            authPreferences.clearLoginState()
            navigateToLogin()
        }

        binding.refresh.setOnClickListener {
            viewModel.getUserData(authPreferences.getLogin() ?: "")
        }

        binding.scan.setOnClickListener {
            findNavController().navigate(QrScanDestination)
        }
    }

    private fun navigateToLogin() {
        try {
            authPreferences.clearLoginState()
            findNavController().apply {
                popBackStack(LoginDestination, false)
                navigate(LoginDestination)
            }
        } catch (e: Exception) {
            Log.e("MainFragment", "Error navigating to login", e)
        }
    }

    override fun onDestroyView() {
        super.onDestroyView()
        _binding = null
    }
}